# Analytics Parameter

## Overview

Idea behind `AnalyticsParameter` is that each parameter defines its own key and value (both type and its value) instead of `AnalyticsEvent` being responsible for everything

Ultimate goal is a separation of concerns as following:

| AnalyticsParameter                           | AnalyticsEvent                           |
|----------------------------------------------|------------------------------------------|
| Define parameter key                         | Define event name                        |
| Define parameter value                       | Define event destinations                |
| Define additional payload                    | Define needed `AnalyticsParameter` types |
| Create `[String: Any]` from self and payload |                                          |

Where possible, keep `AnalyticsParameter` types inside its parent `AnalyticsEvent` namespace.

## End goal

Report an event defined as

```swift 
let event = PaywallEvent.paywallShown(.bookpoint(stepCount: 2, optionalStepCount:1))
```

in the following way

```json
{
  "name": "PaywallShown",
  "params": {
    "source": "bookpoint",
    "step": 2,
    "optionalStep": 1
  }
}
```

## Usage example

### Step 1: Define `AnalyticsEvent`

```swift

enum PaywallEvent: AnalyticsEvent {

    ...
}
```

### Step 2: Define `AnalyticsParameter types` (in event namespace if possible)


```swift

enum PaywallEvent: AnalyticsEvent {

    struct UStepCount: AnalyticsParameter {

        static let key = "Step"
        let value: UInt
    }

    struct UOptionalStepCount: AnalyticsParameter {

        static let key = "OptionalStep"
        let value: UInt
    }

    /// Example when an AnalyticsParamete contains additional payload
    enum Source: AnalyticsParameter {

        case bookpoint(stepCount: UStepCount, optionalStepCount: UOptionalStepCount?)

        static let key = "Source"

        var value: String {

            switch self {
            case .bookpoint: return "bookpoint"
            }
        }

        var payload: [AnalyticsParameterBuildable?]? {

            switch self {
            case .bookpoint(let stepCount, let optionalStepCount):
                return [stepCount, optionalStepCount]
            }
        }
    } 
}
```

### Step 3: Define events and use `intoDictionary` from `AnalyticsParameter` where needed

```swift

enum PaywallEvent: AnalyticsEvent {

    // AnalyticsParameter definitions
    ...

    case paywallShown(source: Source) // !< Source is AnalyticsParameter
    ...

    var name: String {

        switch self {

            case paywallShown: return "PaywallShown"
            ...
        }
    }

    var parameters: [String: Any]? {

        switch self {

            case paywallShown(let source):
                return source.intoDictionary()
            ...
        }
    }
}

```

---

## Additional tips

### Reporting multiple parameters

when an event is defined as

```swift

enum EventType: AnalyticsEvent {

    ...
    case eventWithTwoParams(param1: Param1, param2: Param2)
    ...
}
```

its parameters should be reported as

```swift
enum EventType: AnalyticsEvent {

    ...
    case eventWithTwoParams(param1: Param1, param2: Param2)
    ...

    var parameters: [String: Any]? {

        switch self {

            ...
            case eventWithTwoParams(let param1, let param2):
                return [param1, param2].intoDictionary()
            ...
        }
    }
}
```

### ExpressibleBy`<FoundationType>`Literal

Even though we are defining new structs implementing `AnalyticsParameter`, to provide simpler and more transparent 
outside usage some values can implement `ExpressibleBy<FoundationType>Literal`.

In the above example it would look as following:

```swift
extension PaywallEvent.UStepCount: ExpressibleByIntegerLiteral {

    init(integerLiteral: UInt) {
        init(value: integerLitereal)
    }
}
```

which would provide following usage:

```swift
let directlyInitialized = PaywallEvent.UStepCount(value: 3)
let integerInitialized: PaywallEvent.UStepCount = 3
```

or

```swift
func withUStepCountParam(_ param: PaywallEvent.UStepCount) { ... }

withUStepCountParam(4)
```
